#
# U-BOOT TCL File
#
# (C) Copyright 2007 Michal Simek
#
# Michal SIMEK <monstr@monstr.eu>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA
#
# ChangeLOG:
# 3.00.b Repair by David Gibson
# 3.00.a Solving problem with bridges
# 2.00.b Remove problem with storing file
# 2.00.a Testing interrupt connection between Microblaze & INTC
# 1.01.c add simple FDT support - testing version
# 1.01.b add Microblaze HW_VER support, FLAGS, Uart16550. Remove MSS drivers
#	dependence, Timer handle throught INTC, BARREL shifter
# 1.01.a add MSR support and mch_opb_ddr
# 1.01.f add FSL support
# 1.01.e add IIC support
# 1.00.d clear Czech comments
# 1.00.c synchronization FLASH_MEMORY

# BUGS
# handle OPB_BUS

# Globals variable
set drvlist [list]
set cpunumber 0
set periphery_array ""

proc uboot_drc {os_handle} {
	puts "\#--------------------------------------"
	puts "\# uboot BSP DRC...!"
	puts "\#--------------------------------------"
}

proc generate {os_handle} {
	puts "\#--------------------------------------"
	puts "\# FDT BSP generate..."
	puts "\#--------------------------------------"

# Open config.mk file
	set config_file_name2 "config.mk"
	set config_file2 [open $config_file_name2 w]
# Print GPL
	headerm $config_file2

# open config files
	set config_file_name "xparameters.h"
	set config_file [open $config_file_name w]
# print GPL license
	headerc $config_file

	set proc_handle [xget_libgen_proc_handle]
# ******************************************************************************
# print system clock
	set sys_clock [xget_sw_parameter_value $proc_handle "CORE_CLOCK_FREQ_HZ"]
	puts $config_file "/* System Clock Frequency */"
	puts $config_file "#define XILINX_CLOCK_FREQ\t$sys_clock"
	puts $config_file ""

# ******************************************************************************
# FDT - FDT - FDT - FDT - FDT - FDT - FDT - FDT - FDT - FDT - FDT - FDT - FDT
#Open file for FDT
	set fdt_file "system.dts"
	set fdt [open $fdt_file w]
	set l "1"

#preinit part
	headerc $fdt
	puts $fdt "[ts $l]/ {"
	puts $fdt "[tt $l]model = \"mONStR\";"
	puts $fdt "[tt $l]compatible = \"ibm,opb\";"
	puts $fdt "[tt $l]#address-cells = <1>;"
	puts $fdt "[tt $l]#size-cells = <1>;"

#standart part
	set l [expr $l + 1]

#-------------------------------------------------------
	set hwproc_handle [xget_handle $proc_handle "IPINST"]
	set proctype [xget_value $hwproc_handle "OPTION" "IPNAME"]
	switch $proctype {
		"microblaze" {
			set intc [get_handle_to_intc $proc_handle "Interrupt"]
#generate chosen part - because I know what is interrupt contoller
			puts $fdt "[ts $l]chosen \{"
			if { [llength $intc] != 0 } {
				set base [xget_hw_parameter_handle $intc "C_BASEADDR"]
#FIXME - it is neccessary have full path - not only from ROOT
				puts $fdt "[tt $l]intc = \"/[string map -nocase {"_" "-"} [xget_hw_value $intc]]@[format "%x" [xget_hw_value $base]]\";"
				unset base
			}
			set timer [xget_sw_parameter_value $os_handle "timer"]
			set timer [xget_sw_ipinst_handle_from_processor $proc_handle $timer]
			if { [llength $timer] != 0 } {
				set base [xget_hw_parameter_handle $timer "C_BASEADDR"]
#FIXME - it is neccessary have full path - not only from ROOT
				puts $fdt "[tt $l]timer = \"/[string map -nocase {"_" "-"} [xget_hw_value $timer]]@[format "%x" [xget_hw_value $base]]\";"
				unset base
			}
			unset timer
			set uart [xget_sw_parameter_value $os_handle "stdin_out"]
			set uart [xget_sw_ipinst_handle_from_processor $proc_handle $uart]
			if { [llength $uart] != 0 } {
				set base [xget_hw_parameter_handle $uart "C_BASEADDR"]
#FIXME - it is neccessary have full path - not only from ROOT
				puts $fdt "[tt $l]console = \"/serial@[format "%x" [xget_hw_value $base]]\";"
				unset base
			}
			unset uart
			set bootargs [xget_sw_parameter_value $os_handle "bootargs"]
			puts $fdt "[tt $l]bootargs = \"$bootargs\";"
			puts $fdt "[tt $l]/*"
			puts $fdt "[tt $l]	root=/dev/mtdblock0 console=ttyS0,115200"
			puts $fdt "[tt $l]	bootargs = \"root=/dev/ram quiet"
			puts $fdt "[tt $l]	root=/dev/nfs rw nfsroot=192.168.0.5:/tftpboot/nfs ethaddr=00:01:02:03:04:05"
			puts $fdt "[tt $l]	ip=192.168.0.4:192.168.0.5:192.168.0.5:255.255.255.0:idu:off"
			puts $fdt "[tt $l]*/"
			unset bootargs
			puts $fdt "[ts $l]\} ;"
#------------------------------------------------------------------------------------
			puts $fdt "[gen_microblaze $hwproc_handle $l]"
#			puts $fdt "[ts $l]opb_v20 \{"
#			puts $fdt "[tt $l]compatible = \"xlnx,opb-v20\",\"ibm,opb\"";
#			set l [expr $l + 1]

			puts $fdt "[bus_bridge $hwproc_handle $l $intc "DOPB"]"
			puts "ok"
			puts $fdt "[bus_bridge $hwproc_handle $l $intc "DPLB"]"
#			set l [expr $l - 1]
#			puts $fdt "[ts $l]\} ;"
		}
		"ppc405" -
		"ppc405_virtex4" {
			set intc [get_handle_to_intc $proc_handle "EICC405EXTINPUTIRQ"]
#generate chosen part
			puts $fdt "[ts $l]chosen \{"
			set bootargs [xget_sw_parameter_value $os_handle "bootargs"]
			puts $fdt "[tt $l]bootargs = \"$bootargs\";"
			unset bootargs
			puts $fdt "[ts $l]\} ;"
#end of generation chosen part
			puts $fdt "[gen_ppc405 $hwproc_handle $l]"
			puts $fdt "[bus_bridge $hwproc_handle $l $intc "DPLB"]"
		}
		default {
			error "unsupported CPU"
		}
	}
#last part
	set l [expr $l - 1]
	puts $fdt "[ts $l]\} ;"
	close $fdt
#	error "end of FDT generation"

# ******************************************************************************
#
# U-BOOT part
#
# ******************************************************************************
	puts "\#--------------------------------------"
	puts "\# uboot BSP generate..."
	puts "\#--------------------------------------"
# Microblaze
	set hwproc_handle [xget_handle $proc_handle "IPINST"]
	set args [xget_hw_parameter_handle $hwproc_handle "*"]
	set proctype [xget_value $hwproc_handle "OPTION" "IPNAME"]
	switch $proctype {
		"microblaze" {
# write only name of instance
		foreach arg $args {
			set arg_name [xget_value $arg "NAME"]
			set arg_name [string map -nocase {C_ ""} $arg_name]
			set arg_value [xget_value $arg "VALUE"]
#			puts "DEBUG $arg_name $arg_value"
			switch -regexp $arg_name {
				INSTANCE {
					puts $config_file "/* Microblaze is $arg_value */"
					set mr_name $arg_value
					break;
				}
				default {}
			}
		}
		foreach arg $args {
			set arg_name [xget_value $arg "NAME"]
			set arg_name [string map -nocase {C_ ""} $arg_name]
			set arg_value [xget_value $arg "VALUE"]
#			puts $config_file3 "DEBUG $arg_name $arg_value"
			switch $arg_name {
				USE_MSR_INSTR {
					puts $config_file "#define XILINX_USE_MSR_INSTR\t$arg_value"
				}
				FSL_LINKS {
					puts $config_file "#define XILINX_FSL_NUMBER\t$arg_value"
				}
				HW_VER {
					puts $config_file2 "PLATFORM_CPPFLAGS += -mcpu=v$arg_value"
				}
				USE_BARREL {
					if {[string match $arg_value "1"]} {
						puts $config_file2 "PLATFORM_CPPFLAGS += -mxl-barrel-shift"
					}
				}
				USE_DIV {
					if {[string match $arg_value "1"]} {
						puts $config_file2 "PLATFORM_CPPFLAGS += -mno-xl-soft-div"
					}
#FIXME What is -mno-xl-hard-div
				}
				USE_HW_MUL {
					if {[string match $arg_value "1"]} {
						puts $config_file2 "PLATFORM_CPPFLAGS += -mno-xl-soft-mul"
					} else {
						puts $config_file2 "PLATFORM_CPPFLAGS += -mxl-soft-mul"
					}
#FIXME What is -mxl-multiply-high???
				}
				USE_PCMP_INSTR {
					if {[string match $arg_value "1"]} {
						puts $config_file2 "PLATFORM_CPPFLAGS += -mxl-pattern-compare"
					}
				}
				USE_FPU {
					if {[string match $arg_value "1"]} {
						puts $config_file2 "PLATFORM_CPPFLAGS += -mhard-float"
					}
				}
				PVR {
					puts $config_file "#define XILINX_PVR\t$arg_value"
				}
				default {}
			}
		}

# OPB bus resolve
		set dopb2 [xget_handle $hwproc_handle "BUS_INTERFACE" "DOPB"]
		set dopb [xget_value $dopb2 "VALUE"]
		set iopb [xget_handle $hwproc_handle "BUS_INTERFACE" "IOPB"]
		set iopb [xget_value $iopb "VALUE"]
		if { $dopb == $iopb } {
		set system_bus "$dopb"
		puts "System bus for instruction and data $dopb"

#testing

#		set bus [xget_sw_parameter_value $os_handle "opb_v20"]
##		set bus_handle [xget_sw_ipinst_handle_from_processor $proc_handle $bus]
##		set hodn [xget_sw_parameter_value $bus_handle "C_EXT_RESET_HIGH"]
#		puts "fdf $bus fds"
#		set clk [xget_handle $dopb "PORT" "OPB_Clk"]
#		puts "$clk"
#		set clk [xget_value $clk "VALUE"]
#		error "$clk"
#end testing

		} else {
			error "different microblaze architecture - dual busses $iopb $dopb"
		}
	}
	"ppc405" -
	"ppc405_virtex4" {
		puts "unsupported processor type $proctype\n"
		error "ERROR $proctype not supported by U-BOOT yet"
	}
	default {
		error "This type of CPU is not supported by U-BOOT yet"
	}
	}
	puts $config_file ""
# ******************************************************************************
# Interrupt controler
	set intc [xget_sw_parameter_value $os_handle "intc"]
	if {[string match "" $intc] || [string match -nocase "none" $intc]} {
		puts $config_file "/* Interrupt controller not defined */"
	} else {
		puts $config_file "/* Interrupt controller is $intc */"
		set intc_handle [xget_sw_ipinst_handle_from_processor $proc_handle $intc]
#test for correct system bus
		test_buses $system_bus $intc_handle "SOPB"
# Interrupt controller address
		set intc_base [xget_sw_parameter_value $intc_handle "C_BASEADDR"]
		set intc_end [xget_sw_parameter_value $intc_handle "C_HIGHADDR"]
		set intc_base [format "0x%08x" $intc_base]
		puts $config_file "#define XILINX_INTC_BASEADDR\t$intc_base"
		set intc_value [xget_sw_parameter_value $intc_handle "C_NUM_INTR_INPUTS"]

		puts $config_file "#define XILINX_INTC_NUM_INTR_INPUTS\t$intc_value"
		set intc_value [expr $intc_value - 1]
		set intc_handle [xget_sw_ipinst_handle_from_processor $proc_handle $intc]
		set port_list [xget_port_by_subtype $intc_handle "*"]
		foreach port $port_list {
			set name [xget_value $port "NAME"]
			if {[string match -nocase $name "intr"]} {
				set intc_irq [xget_value $port "VALUE"]
#DEBUG				puts $config_file "/* pripojene interrupty $name=$intc_irq */"
				set intc_signals [split $intc_irq "&"]
				#spit the signals
#DEBUG				puts $config_file "$signals"
			}
		}
		puts $config_file ""

# ******************************************************************************
# Timer part
# handle timer if exists intc
	set timer [xget_sw_parameter_value $os_handle "timer"]
		if {[string match "" $timer] || [string match -nocase "none" $timer]} {
			puts $config_file "/* Timer not defined */"
		} else {
			set timer_handle [xget_sw_ipinst_handle_from_processor $proc_handle $timer]
#test for correct system bus
			test_buses $system_bus $timer_handle "SOPB"
			set timer_base [xget_sw_parameter_value $timer_handle "C_BASEADDR"]
			set timer_end [xget_sw_parameter_value $timer_handle "C_HIGHADDR"]
			set timer_base [format "0x%08x" $timer_base]
			puts $config_file "/* Timer pheriphery is $timer */"
			puts $config_file "#define XILINX_TIMER_BASEADDR\t$timer_base"

#			puts "$timer_handle $intc $intc_value $intc_signals Interrupt"
			set intr [get_intr $timer_handle $intc $intc_value $intc_signals "Interrupt"]
			puts $config_file "#define XILINX_TIMER_IRQ\t$intr"
		}
	}
	puts $config_file ""
# ******************************************************************************
# UartLite driver - I suppose, only uartlite driver
	set uart [xget_sw_parameter_value $os_handle "stdin_out"]
	if {[string match "" $uart] || [string match -nocase "none" $uart]} {
		puts $config_file "/* Uart not defined */"
		puts "ERROR Uart not specified. Please specific console"
	} else {
		set uart_handle [xget_sw_ipinst_handle_from_processor $proc_handle $uart]
		puts $config_file "/* Uart pheriphery is $uart */"
		set type [xget_value $uart_handle "VALUE"]
		switch $type {
			"opb_uart16550" {
#				puts $config_file "#define XILINX_UART16550"
				set uart_baseaddr [xget_sw_parameter_value $uart_handle "C_BASEADDR"]
				set uart_baseaddr [format "0x%08x" $uart_baseaddr]
				puts $config_file "#define XILINX_UART16550_BASEADDR\t$uart_baseaddr"
# FIXME bad sysclock frequency
#				puts $config_file "#define XILINX_UART16550_CLOCK_HZ\t$system_bus"
#it is driver dependent code
				set drv_handle [xget_handle $proc_handle "DRIVER" "uartns550"]
				if { [llength $drv_handle] == 0 } {
					error "Please specify your standalone driver for Uart16550"
				}
				puts $config_file "#define XILINX_UART16550_CLOCK_HZ\t[xget_value $drv_handle "PARAMETER" "CLOCK_HZ"]"
#dependent end
			}
			"opb_uartlite" -
			"xps_uartlite" {
				set args [xget_sw_parameter_handle $uart_handle "*"]
				foreach arg $args {
					set arg_name [xget_value $arg "NAME"]
					set arg_value [xget_value $arg "VALUE"]
					set arg_name [string map -nocase {C_ ""} $arg_name]
#DEBUG					puts $config_file "$arg_name $arg_value"
					case $arg_name in {
						"BASEADDR" {
							puts $config_file "#define XILINX_UARTLITE_${arg_name}\t$arg_value"
							set uart_base $arg_value
						}
						"BAUDRATE" {
							puts $config_file "#define XILINX_UARTLITE_${arg_name}\t$arg_value"
						}
						"HIGHADDR" {
							set uart_end $arg_value
						}
						default	{}
					}
				}
			}
			default {
				error "Unsupported type of console - $type"
			}
		}
	}
	puts $config_file ""

# ******************************************************************************
# IIC driver - I suppose, only uartlite driver
	set iic [xget_sw_parameter_value $os_handle "iic"]
	if {[string match "" $iic] || [string match -nocase "none" $iic]} {
		puts $config_file "/* IIC doesn't exist */"
	} else {
		set iic_handle [xget_sw_ipinst_handle_from_processor $proc_handle $iic]
		set iic_baseaddr [xget_sw_parameter_value $iic_handle "C_BASEADDR"]
		set iic_baseaddr [format "0x%08x" $iic_baseaddr]
		set iic_freq [xget_sw_parameter_value $iic_handle "C_IIC_FREQ"]
		set iic_bit [xget_sw_parameter_value $iic_handle "C_TEN_BIT_ADR"]
		puts $config_file "/* IIC pheriphery is $iic */"
		puts $config_file "#define XILINX_IIC_0_BASEADDR\t$iic_baseaddr"
		puts $config_file "#define XILINX_IIC_0_FREQ\t$iic_freq"
		puts $config_file "#define XILINX_IIC_0_BIT\t$iic_bit"
	}
	puts $config_file ""
# ******************************************************************************
# GPIO configuration
	set gpio [xget_sw_parameter_value $os_handle "gpio"]
	if {[string match "" $gpio] || [string match "none" $gpio]} {
		 puts $config_file "/* GPIO doesn't exist */"
	} else {
		set base_param_name [format "C_BASEADDR" $gpio]
		set gpio_handle [xget_sw_ipinst_handle_from_processor $proc_handle $gpio]
		set gpio_base [xget_sw_parameter_value $gpio_handle $base_param_name]
		set gpio_base [format "0x%08x" $gpio_base]

		set gpio_end [xget_sw_parameter_value $gpio_handle "C_HIGHADDR"]
		set gpio_end [format "0x%08x" $gpio_end]


		puts $config_file "/* GPIO is $gpio*/"
		puts $config_file "#define XILINX_GPIO_BASEADDR\t$gpio_base"
	}
	puts $config_file ""
# ******************************************************************************
# System memory
	set main_mem [xget_sw_parameter_value $os_handle "main_memory"]
	if {[string match "" $main_mem] || [string match "none" $main_mem]} {
		puts "ERROR main_memory not specified. Please specific main_memory"
		puts $config_file "/* Main Memory doesn't exist */"
	} else {
		set main_mem_bank [xget_sw_parameter_value $os_handle "main_memory_bank"]
		set main_mem_handle [xget_sw_ipinst_handle_from_processor $proc_handle $main_mem]
		if {[string compare -nocase $main_mem_handle ""] != 0} {

			if {[xget_hw_value $main_mem_handle] == "mpmc"} {
				set base_param_name "C_MPMC_BASEADDR"
				set high_param_name "C_MPMC_HIGHADDR"
			} else {
				set base_param_name [format "C_MEM%i_BASEADDR" $main_mem_bank]
				set high_param_name [format "C_MEM%i_HIGHADDR" $main_mem_bank]
			}

			set eram_base [xget_sw_parameter_value $main_mem_handle $base_param_name]
			set eram_end [xget_sw_parameter_value $main_mem_handle $high_param_name]
			set eram_size [expr $eram_end - $eram_base + 1]
			set eram_base [format "0x%08x" $eram_base]
			set eram_size [format "0x%08x" $eram_size]
			set eram_high [expr $eram_base + $eram_size]
			set eram_high [format "0x%08x" $eram_high]
			puts $config_file "/* Main Memory is $main_mem */"
			puts $config_file "#define XILINX_RAM_START\t$eram_base"
			puts $config_file "#define XILINX_RAM_SIZE\t$eram_size"
		}
	}
	puts $config_file ""
# ******************************************************************************
# Flash memory
	set flash_mem [xget_sw_parameter_value $os_handle "flash_memory"]
	if {[string match "" $flash_mem] || [string match "none" $flash_mem]} {
		 puts $config_file "/* FLASH doesn't exist $flash_mem */"
		 puts "FLASH doesn't exists"
	} else {
		set flash_mem_handle [xget_sw_ipinst_handle_from_processor $proc_handle $flash_mem]
		set flash_mem_bank [xget_sw_parameter_value $os_handle "flash_memory_bank"]
		set base_param_name [format "C_MEM%i_BASEADDR" $flash_mem_bank]
		set high_param_name [format "C_MEM%i_HIGHADDR" $flash_mem_bank]
		set flash_start [xget_sw_parameter_value $flash_mem_handle $base_param_name]
		set flash_end [xget_sw_parameter_value $flash_mem_handle $high_param_name]
		set flash_size [expr $flash_end - $flash_start + 1]
		set flash_start [format "0x%08x" $flash_start]
		set flash_size [format "0x%08x" $flash_size]
		if {$eram_base < $flash_start} {
			puts $config_file "/* Flash Memory is $flash_mem */"
			puts $config_file "#define XILINX_FLASH_START\t$flash_start"
			puts $config_file "#define XILINX_FLASH_SIZE\t$flash_size"
		} else {
			error "Flash base address must be on higher address than ram memory"
		}
	}
	puts $config_file ""
# ******************************************************************************
# Sysace
	set sysace [xget_sw_parameter_value $os_handle "sysace"]
	if {[string match "" $sysace] || [string match "none" $sysace]} {
		puts $config_file "/* Sysace doesn't exist */"
	} else {
		puts $config_file "/* Sysace Controller is $sysace */"
		set sysace_handle [xget_sw_ipinst_handle_from_processor $proc_handle $sysace]
		set args [xget_sw_parameter_handle $sysace_handle "*"]
		foreach arg $args {
			set arg_name [xget_value $arg "NAME"]
			set arg_name [string map -nocase {C_ ""} $arg_name]
			set arg_value [xget_value $arg "VALUE"]
#			puts $config_file "$arg_name $arg_value"
			switch $arg_name {
				"BASEADDR" {
#					set arg_value [xget_value $arg "VALUE"]
					puts $config_file "#define XILINX_SYSACE_${arg_name}\t$arg_value"
					set sysace_base $arg_value
				}
				"MEM_WIDTH" {
#					set arg_value [xget_value $arg "VALUE"]
					puts $config_file "#define XILINX_SYSACE_${arg_name}\t$arg_value"
				}
				"HIGHADDR" {
#					set arg_value [xget_value $arg "VALUE"]
					set sysace_end $arg_value
				}
				"HW_VER" {
#					puts "revize $arg_value"
				}
				default {}
			}
		 }
	}
	puts $config_file ""
# ******************************************************************************
# Ethernet
	set ethernet [xget_sw_parameter_value $os_handle "ethernet"]
	if {[string match "" $iic] || [string match -nocase "none" $ethernet]} {
		puts $config_file "/* Ethernet doesn't exist */"
	} else {
		set ethernet_handle [xget_sw_ipinst_handle_from_processor $proc_handle $ethernet]
		set ethernet_name [xget_value $ethernet_handle "VALUE"]
#		puts "DEBUG $ethernet_name"
		puts $config_file "/* Ethernet controller is $ethernet */"

		switch $ethernet_name {
			"opb_ethernet" {
				set args [xget_sw_parameter_handle $ethernet_handle "*"]
				foreach arg $args {
					set arg_name [xget_value $arg "NAME"]
					set arg_value [xget_value $arg "VALUE"]
					set arg_name [string map -nocase {C_ ""} $arg_name]
#DEBUG					puts $config_file "$arg_name $arg_value"
					case $arg_name in {
						{"MII_EXIST" "DMA_PRESENT" "HALF_DUPLEX_EXIST"} {
							puts $config_file "#define XILINX_EMAC_${arg_name}\t$arg_value"
						}
						"BASEADDR" {
							puts $config_file "#define XILINX_EMAC_${arg_name}\t$arg_value"
							set ethernet_base $arg_value
						}
						"HIGHADDR" {
							set ethernet_end $arg_value
						}

						default {}
					}
				}
			}
			"xps_ll_temac" {
				set args [xget_sw_parameter_handle $ethernet_handle "*"]
				foreach arg $args {
					set arg_name [xget_value $arg "NAME"]
					set arg_value [xget_value $arg "VALUE"]
					set arg_name [string map -nocase {C_ ""} $arg_name]
#DEBUG					puts $config_file "$arg_name $arg_value"
					case $arg_name in {
						{"MII_EXIST" "DMA_PRESENT" "HALF_DUPLEX_EXIST"} {
							puts $config_file "#define XILINX_LLEMAC_${arg_name}\t$arg_value"
						}
						"BASEADDR" {
							puts $config_file "#define XILINX_LLEMAC_${arg_name}\t$arg_value"
							set ethernet_base $arg_value
						}
						"HIGHADDR" {
							set ethernet_end $arg_value
						}

						default {}
					}
				}
			}
			"opb_ethernetlite" {
				set args [xget_sw_parameter_handle $ethernet_handle "*"]
				foreach arg $args {
					set arg_name [xget_value $arg "NAME"]
					set arg_value [xget_value $arg "VALUE"]
					set arg_name [string map -nocase {C_ ""} $arg_name]
#DEBUG					puts $config_file "$arg_name $arg_value"
					case $arg_name in {
						{"BASEADDR" "TX_PING_PONG" "RX_PING_PONG"} {
							puts $config_file "#define XILINX_EMACLITE_${arg_name}\t$arg_value"
						}
						default {}
					}
				}
			}
			default {
				error "Unsupported ethernet periphery - $ethernet_name"
			}
		}
	}
#*******************************************************************************
# U-BOOT position in memory
	set text_base [xget_sw_parameter_value $os_handle "uboot_position"]
	set text_base [format "0x%08x" $text_base]
	puts $config_file2 ""
	if {$text_base == 0} {
		if {[llength $eram_base] != 0 } {
			puts $config_file2 "TEXT_BASE = $eram_base"
			puts "WARNING automatic U-BOOT position"
		} else {
			error "Main memory is not defined"
		}
	} elseif {![catch {unset $eram_base}]} {
		if {$eram_base < $text_base && $eram_high > $text_base} {
#			puts $config_file2 "# TEXT BASE "
			puts $config_file2 "TEXT_BASE = $text_base"
			puts $config_file2 ""
# print system clock
#			set sw [xget_sw_parameter_value $proc_handle "HW_INSTANCE"]

# FIXME Parameters for Microblaze from MHS files - look at Microblaze SW manual
# print microblaze params
#			set hwproc_handle [xget_handle $proc_handle "IPINST"]
#			set args [xget_hw_parameter_handle $hwproc_handle "*"]
#			set proctype [xget_value $hwproc_handle "OPTION" "IPNAME"]
		} else {
		error "ERROR u-boot position is out of range $eram_base - $eram_high"
		}
	}

	puts "}"



	close $config_file
	close $config_file2
#*******************************************************************************
}

##################################################
# procedure post_generate
# This generates the drivers directory for uboot
# and runs the ltypes script
##################################################

proc post_generate {lib_handle} {
	set pwd [pwd]
	set name [xget_value $lib_handle "PARAMETER" name]
	if {$name != ""} {
		set uboot_dir "uboot/board/xilinx/$name"
		exec bash -c "mkdir -p $uboot_dir"
		exec bash -c "cp ../../include/xparameters.h $uboot_dir/xparameters-edk.h"
		exec bash -c "cp ../../../system.mhs $uboot_dir"
		exec bash -c "cp ../../../system.mss $uboot_dir"
		exec bash -c "mv xparameters.h $uboot_dir"
		exec bash -c "mv config.mk $uboot_dir"
	}
}

proc headerm {config_file2} {
	puts $config_file2 "\#"
	puts $config_file2 "\# (C) Copyright 2007 Michal Simek"
	puts $config_file2 "\#"
	puts $config_file2 "\# Michal SIMEK <monstr@monstr.eu>"
	puts $config_file2 "\#"
	puts $config_file2 "\# This program is free software; you can redistribute it and/or"
	puts $config_file2 "\# modify it under the terms of the GNU General Public License as"
	puts $config_file2 "\# published by the Free Software Foundation; either version 2 of"
	puts $config_file2 "\# the License, or (at your option) any later version."
	puts $config_file2 "\#"
	puts $config_file2 "\# This program is distributed in the hope that it will be useful,"
	puts $config_file2 "\# but WITHOUT ANY WARRANTY; without even the implied warranty of"
	puts $config_file2 "\# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the"
	puts $config_file2 "\# GNU General Public License for more details."
	puts $config_file2 "\#"
	puts $config_file2 "\# You should have received a copy of the GNU General Public License"
	puts $config_file2 "\# along with this program; if not, write to the Free Software"
	puts $config_file2 "\# Foundation, Inc., 59 Temple Place, Suite 330, Boston,"
	puts $config_file2 "\# MA 02111-1307 USA"
	puts $config_file2 "\#"
	puts $config_file2 "\# CAUTION: This file is automatically generated by libgen."
	puts $config_file2 "\# Version: [xget_swverandbld]"
	puts $config_file2 "\#"
	puts $config_file2 ""
}

proc headerc {config_file2} {
	puts $config_file2 "/*"
	puts $config_file2 " * (C) Copyright 2007 Michal Simek"
	puts $config_file2 " *"
	puts $config_file2 " * Michal SIMEK <monstr@monstr.eu>"
	puts $config_file2 " *"
	puts $config_file2 " * This program is free software; you can redistribute it and/or"
	puts $config_file2 " * modify it under the terms of the GNU General Public License as"
	puts $config_file2 " * published by the Free Software Foundation; either version 2 of"
	puts $config_file2 " * the License, or (at your option) any later version."
	puts $config_file2 " *"
	puts $config_file2 " * This program is distributed in the hope that it will be useful,"
	puts $config_file2 " * but WITHOUT ANY WARRANTY; without even the implied warranty of"
	puts $config_file2 " * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the"
	puts $config_file2 " * GNU General Public License for more details."
	puts $config_file2 " *"
	puts $config_file2 " * You should have received a copy of the GNU General Public License"
	puts $config_file2 " * along with this program; if not, write to the Free Software"
	puts $config_file2 " * Foundation, Inc., 59 Temple Place, Suite 330, Boston,"
	puts $config_file2 " * MA 02111-1307 USA"
	puts $config_file2 " *"
	puts $config_file2 " * CAUTION: This file is automatically generated by libgen."
	puts $config_file2 " * Version: [xget_swverandbld]"
	puts $config_file2 " */"
	puts $config_file2 ""
}


# from xillib_hw.tcl
#
# Return the handle to the bus interface associated with this address parameter.
# Parameters: periph_handle: peripheral handle
#	addr_name: name of an address parameter
#
proc xget_addrparam_busif {periph_handle addr_param_name} {
	set baseaddr_handle [xget_handle $periph_handle "PARAMETER" $addr_param_name]
# test on null parameter
	if { [llength $baseaddr_handle] == 0 } {
		return ""
	}

	set baseaddr_bushandle [xget_handle $baseaddr_handle "SUBPROPERTY" "BUS"]
	puts "DEBUG $baseaddr_bushandle =="
	if { [llength $baseaddr_bushandle] == 0 } {
	# See if this address has a pair
		if {[string match *_BASEADDR $addr_param_name]} {
			set addr_pair [string range $addr_param_name 0 [expr [string last "_BASEADDR" $addr_param_name] -1]]
			append addr_pair "_HIGHADDR"
		} elseif {[string match *_HIGHADDR $addr_param_name]} {
			set addr_pair [string range $addr_param_name 0 [expr [string last "_HIGHADDR" $addr_param_name] -1]]
			append addr_pair "_BASEADDR"
		} else {
			return ""
		}
		set baseaddr_handle [xget_handle $periph_handle "PARAMETER" $addr_pair]
		set baseaddr_bushandle [xget_handle $baseaddr_handle "SUBPROPERTY" "BUS"]
		if { [llength $baseaddr_bushandle] == 0 } {
			return ""
		}
	}
	set baseaddr_busname [xget_value $baseaddr_handle "SUBPROPERTY" "BUS"]
#	set baseaddr_busname [xget_value $baseaddr_bushandle "VALUE"]
	set busif_handle [xget_handle $periph_handle "BUS_INTERFACE" $baseaddr_busname]
	return $busif_handle
}

#test for peripheral - if is correct setting system bus
proc test_buses {system_bus handle bus_type} {
	set bus [xget_handle $handle "BUS_INTERFACE" $bus_type]
	if { [llength $bus] == 0 } {
		return 1
	}
	set bus [xget_value $bus "VALUE"]
	if { $bus != $system_bus} {
		error "Periphery $handle is connected to another system bus $bus ----"
		return 0
	} else {
		set name [xget_value $handle "NAME"]
		puts "$name has correct system_bus $system_bus"
	}
	return 1
}

#testovaci procedury

proc xdefine_include_file2 {drv_handle file_name drv_string args} {
	foreach arg $args {
		set value [xget_value $drv_handle "PARAMETER" $arg]
		if {[llength $value] == 0} {
			error "Zero size of PARAMETER. Please append argument args."
		} else {
#			puts $file_name "#defineoo [xget_dname $drv_string $arg] [xget_value $drv_handle "PARAMETER" $arg]"
			return [xget_value $drv_handle "PARAMETER" $arg]
		}
	}
}

#get interrupt count
proc get_intr {per_handle intc intc_value intc_signals port_name} {
			 if {![string match "" $intc] && ![string match -nocase "none" $intc]} {
				set port_list [xget_port_by_subtype $per_handle "*"]
				foreach port $port_list {
						set name [xget_value $port "NAME"]
#DEBUG						puts "name:$name = $port" # finding proper port
						if {[string match -nocase $name $port_name]} {
							set value [xget_value $port "VALUE"]
							#correct name of interrupt wire
#DEBUG							puts "value=$value"
							set delka $intc_value
							foreach i $intc_signals {
#							FIXME I don't know what exactly do
								set i [string trim $i]
#								puts "delka=$delka $i $value"
								if {[string match -nocase $i $value]} {
									return $delka
								}
#DEBUG								else {puts "nesedi"}
								set delka [expr $delka - 1]
							}
						}
					}

#FAIL STATE
			return -1;
	}
}







proc bridge {slave level front} {
	set out ""
	set par_handle [xget_hw_parameter_handle $slave *]
	set name [xget_hw_name $slave]
	set value [xget_hw_value $slave]


	set count [xget_hw_parameter_value $slave "C_NUM_ADDR_RNG"]
	if { [llength $count] == 0 } {
		set count 1
	}
#	puts "$name - $count"
	for {set x 0} {$x < $count} {incr x} {
		set baseaddr [xget_hw_parameter_value $slave "$front$x\_BASEADDR"]
		set highaddr [xget_hw_parameter_value $slave "$front$x\_HIGHADDR"]

		set size [expr $highaddr - $baseaddr + 1]
		if { $size < 0 } {
			error "Bad highaddr for $name, $value"
		}
		set baseaddr [format "%x" $baseaddr]
		set size [format "%x" $size]
		if {$x == 0} {
			set out "$out[ts $level]$value@$baseaddr \{"
		}
#		set out "$out\n[tt $level]ranges = < $x $baseaddr $size >;"
	}
	return $out
}





#irq -	0 - don't generate irq number
#	1 - generate irq number
#level - indentation - number of tab
proc slaveip {slave level irq param dev_type} {
	set out ""
	set par_handle [xget_hw_parameter_handle $slave *]
	set name [xget_hw_name $slave]
	set value [xget_hw_value $slave]

		foreach par $par_handle {
			set par_name [xget_hw_name $par]
			set par_value [xget_hw_value $par]
			switch $par_name {
				"C_BASEADDR" {
					set baseaddr $par_value
#					puts "$par_name $par_value"
				}
				"C_HIGHADDR" {
					set highaddr $par_value
#					puts "$par_name $par_value"
				}
				"HW_VER" {
					set hw_ver $par_value
#					puts "$par_name $par_value"
				}
				default {
#					set out "$out-----name = \"$par_name\"=$par_value; $param\n"
#					puts "$par_name"
				}
			}
		}
	if { ![llength $baseaddr] || ![llength $highaddr] } {
		error "Bad address range $name, $value"
	}
	set size [expr $highaddr - $baseaddr + 1]
	if { $size < 0 } {
		error "Bad highaddr for $name, $value"
	}
	set baseaddr [format "%x" $baseaddr]
	set size [format "%x" $size]

	set value [string map -nocase {"_" "-"} $value]
	switch $dev_type {
		"serial" {
			set out "$out[ts $level]$dev_type@$baseaddr \{\n"
		}
		"network" {
			set out "$out[ts $level]ethernet@$baseaddr \{\n"
		}
		default {
			set out "$out[ts $level]$value@$baseaddr \{\n"
		}
	}

	set ver "$value\_$hw_ver"
	set ver [string map -nocase {"_" "-"} $ver]
	set out "$out[tt $level]compatible = \"xlnx,$ver\",\"xlnx,$value\";\n"
#	set out "$out[tt $level]interrupt-parent = <&interrupt-controller-phandle>;\n"
#	set out "$out[tt $level]name = \"$name\";\n"

	if {![string match $irq "-1"]} {
#for more interrupt sources from periphery
		if {[llength $irq] > 1} {
			foreach i $irq {
				if {![string match $i "-1"]} {
					set out "$out[tt $level]interrupts = < $i 0 >;\n"
				}
			}
		} else {
			set out "$out[tt $level]interrupts = < $irq 0 >;\n"
		}
	} else {
#		puts "don't generate interrupt"
	}
	set out "$out[tt $level]reg = < $baseaddr $size >;\n"
	if {[llength $dev_type]} {
		set out "$out[tt $level]device_type = \"$dev_type\";\n"
	}
#-----------------------------
# generating additional parameters
	if {[llength $param]} {
		if {[llength $param] > 0} {
			foreach i $param {
				set par_handle [xget_hw_parameter_handle $slave "C_$i"]
				set par_name [xget_hw_name $par_handle]
				set par_name [string map -nocase {"C_" ""} $par_name]
				set par_name [string tolower $par_name]
				set par_name [string map -nocase {"_" "-"} $par_name]
				set par_value [xget_hw_value $par_handle]
				set par_value [format "%x" $par_value]
				set out "$out[tt $level]xlnx,$par_name = <$par_value>;\n"
			}
		}
	} else {
#		set out "$out parameters don't exist [llength $param]\n"
	}
#-----------------------------

	set out "$out[ts $level]\} ;"
	return $out
}

proc slaveip_intr {slave level intc ttt param dev_type} {
		set pocet [xget_hw_parameter_value $intc "C_NUM_INTR_INPUTS"]
		set pocet [expr $pocet - 1]
		set intr [xget_hw_port_value $intc "intr"]
		set intr [split $intr "&"]
		set opb_intc [xget_hw_name $intc]
#		puts "---$slave $opb_intc $pocet $intr [llength $ttt]---"

		if {[llength $ttt] > 1} {
			set sig ""
			foreach in $ttt {
				set sig "$sig [get_intr $slave $opb_intc $pocet $intr $in]"
#				puts "$sig"
			}
		} else {
			set sig [get_intr $slave $opb_intc $pocet $intr $ttt]
#			puts "interrupt is $sig"
		}
		slaveip $slave $level $sig $param $dev_type
}

#
#get handle to interrupt controller from CPU handle
#
proc get_handle_to_intc {proc_handle port_name} {
#one CPU handle
	set hwproc_handle [xget_handle $proc_handle "IPINST"]
#hangle to mhs file
	set mhs_handle [xget_hw_parent_handle $hwproc_handle]
#get handle to interrupt port on Microblaze
	set intr_port [xget_value $hwproc_handle "PORT" $port_name]
	if { [llength $intr_port] == 0 } {
		error "CPU has not connection to Interrupt controller"
	}
#	set sink_port [xget_hw_connected_ports_handle $mhs_handle $intr_port "sink"]
#	set sink_name [xget_hw_name $sink_port]
#get source port periphery handle - on interrupt controller
	set source_port [xget_hw_connected_ports_handle $mhs_handle $intr_port "source"]
#get interrupt controller handle
	set intc [xget_hw_parent_handle $source_port]
#	set name [xget_hw_name $intc]
#	puts "$intc $name"
	return $intc
}

proc ts {number} {
	return [tt [expr $number -1]]
}

#retun number of tabulator
proc tt {number} {
	set tab ""
	for {set x 0} {$x < $number} {incr x} {
		set tab "$tab\t"
#		puts ".$tab..."
	}
	return $tab
}


proc gener_slave {slave l intc} {
	set out ""
		set name [xget_hw_name $slave]
		set value [xget_hw_value $slave]
#		puts "$slave $name=$value"
		switch $value {
			"opb_hwicap" -
			"opb2dcr_bridge" -
			"opb_intc" -
			"opb_onewire" {
#don't generate interrupt lines
				set out [slaveip $slave $l -1 "" ""]
			}
			"opb_intc" -
			"xps_intc" {
#don't generate interrupt lines
				set out [slaveip $slave $l -1 "NUM_INTR_INPUTS" ""]
			}
			"opb_mdm" -
			"mdm" {
				set out [slaveip_intr $slave $l $intc "Interrupt" "MB_DBG_PORTS UART_WIDTH USE_UART WRITE_FSL_PORTS" ""]
			}
			"opb_ac97" {
				set out [slaveip_intr $slave $l $intc "Interrupt" "" ""]
			}
			"opb_uartlite" {
				set out [slaveip_intr $slave $l $intc "Interrupt" "BAUDRATE DATA_BITS CLK_FREQ ODD_PARITY USE_PARITY" "serial"]
			}
			"xps_uartlite" {
				set out [slaveip_intr $slave $l $intc "Interrupt" "BAUDRATE DATA_BITS SPLB_CLK_FREQ_HZ ODD_PARITY USE_PARITY" "serial"]
			}
			"opb_timer" -
			"xps_timer" {
				set out [slaveip_intr $slave $l $intc "Interrupt" "COUNT_WIDTH ONE_TIMER_ONLY" ""]
			}
			"opb_sysace" -
			"xps_sysace" {
				set out [slaveip_intr $slave $l $intc "SysACE_IRQ" "MEM_WIDTH" ""]
			}
			"plb_gpio" -
			"opb_gpio" -
			"xps_gpio" {
				set out [slaveip_intr $slave $l $intc "IP2INTC_Irpt" "GPIO_WIDTH IS_DUAL ALL_INPUTS IS_BIDIR" ""]
			}
			"opb_iic" -
			"xps_iic" -
			"opb_spi" {
				set out [slaveip_intr $slave $l $intc "IP2INTC_Irpt" "" ""]
			}
			"opb_can" {
				set out [slaveip_intr $slave $l $intc "IP2Bus_IntrEvent" "" ""]
			}
			"plb_uart16550" -
			"opb_uart16550" -
			"xps_uart16550" {
				set out [slaveip_intr $slave $l $intc "IP2INTC_Irpt" "" "serial"]
			}
			"opb_ethernetlite" {
				set out [slaveip_intr $slave $l $intc "IP2INTC_Irpt" "DUPLEX RX_PING_PONG TX_PING_PONG" "network"]
			}
			"plb_ethernet" -
			"plb_gemac" -
			"plb_temac" {
				set out [slaveip_intr $slave $l $intc "IP2INTC_Irpt" "" "network"]
			}
			"xps_ll_temac" {
				set out [slaveip_intr $slave $l $intc "TemacIntc0_Irpt" "" "network"]
			}
			"opb_ethernet" {
				set out [slaveip_intr $slave $l $intc "IP2INTC_Irpt" "CAM_EXIST DEV_BLK_ID DEV_MIR_ENABLE DMA_PRESENT
				INCLUDE_DEV_PENCODER IPIF_RDFIFO_DEPTH IPIF_WRFIFO_DEPTH MII_EXIST JUMBO_EXIST ERR_COUNT_EXIST
				TX_DRE_TYPE RX_DRE_TYPE TX_INCLUDE_CSUM RX_INCLUDE_CSUM" "network"]
			}
			"opb_ps2_dual_ref" {
				set out [slaveip_intr $slave $l $intc "Sys_Intr1 Sys_Intr2" "" ""]
			}
			"opb_ac97_controller_ref" {
				set out [slaveip_intr $slave $l $intc "Playback_Interrupt Record_Interrupt" "" ""]
			}
			"opb_timebase_wdt" {
				set out [slaveip_intr $slave $l $intc "Timebase_Interrupt" "" ""]
			}
			"opb_btn_decoder" {
				set out [slaveip_intr $slave $l $intc "BTN_decoder_Irpt" "" ""]
			}
			"opb_sdram" {
				set out [memory $slave $l ""]
			}
			"opb_cypress_usb" -
			"plb_ddr" -
			"opb_ddr" -
			"mch_opb_ddr" -
			"opb_emc" -
			"mch_opb_emc" -
			"xps_mch_emc" {
				set out [memory $slave $l "MEM%i_"]
			}
			"mpmc" {
				set out [memory $slave $l "MPMC_"]
			}
			"opb2plb_bridge" {
#				puts "opb2plb_bridge dostal jsem se sem"
				set out [bridge $slave $l "C_RNG"]
				set l [expr $l + 1]
				set out "$out[bus_bridge $slave $l $intc "MPLB"]"
				set l [expr $l - 1]
				set out "$out\n[ts $l]\}\;"
#				puts "$r"
			}
			"plb2opb_bridge" {
#				puts "plb2opb_bridge dostal jsem se sem"
				set out [bridge $slave $l "C_RNG"]
				set l [expr $l + 1]
				set out "$out[bus_bridge $slave $l $intc "MOPB"]"
				set l [expr $l - 1]
				set out "$out\n[ts $l]\}\;"
#				puts "$r"
			}
			"opb_opb_lite" {
				set out [bridge $slave $l "C_DEC"]
				set l [expr $l + 1]
				set out "$out[bus_bridge $slave $l $intc "MOPB"]"
				set l [expr $l - 1]
				set out "$out\n[ts $l]\}\;"
			}
			"microblaze" {
				puts "Other Microblaze CPU $name=$value"
				set out [gen_microblaze $slave $l]
			}
			"ppc405" -
			"ppc405_virtex4" {
				puts "Other PowerPC405 CPU $name=$value"
				set out [gen_ppc405 $slave $l]
			}

			default {
				puts "unsupported $name = $value"
			}
		}
		return "$out"
}



proc memory {slave l mem} {
	set out ""
	set name [xget_hw_name $slave]
	set value [xget_hw_value $slave]
	set par [xget_hw_parameter_handle $slave *]
#	foreach pa $par {
#		set nam [xget_hw_name $pa]
#		set val [xget_hw_value $pa]
#		puts "$nam=$val"
#	}
	set count [xget_hw_parameter_value $slave "C_NUM_BANKS_MEM"]
	if { [llength $count] == 0 } {
		set count 1
	}
#	puts "$name - $count"
	for {set x 0} {$x < $count} {incr x} {
#		puts "$x"
		set base_param_name [format "C_$mem\BASEADDR" $x]
		set high_param_name [format "C_$mem\HIGHADDR" $x]
#		puts "$base_param_name $high_param_name"
		set eram_base [xget_hw_parameter_value $slave $base_param_name]
		set eram_end [xget_hw_parameter_value $slave $high_param_name]
		if {[string match $eram_end "0x00000000"]} {
#			puts "$eram_base $eram_end"
			continue
		}
		set eram_size [expr $eram_end - $eram_base + 1]
		set eram_base [format "0x%08x" $eram_base]
		set eram_size [format "0x%08x" $eram_size]
		set eram_high [expr $eram_base + $eram_size]
		set eram_high [format "0x%08x" $eram_high]
		set eram_base [format "%x" $eram_base]
		set eram_size [format "%x" $eram_size]
		set out "$out[ts $l]memory@$eram_base \{\n"
#		set out "$out[tt $l]edk_name = \"$name\";\n"
#		set out "$out[tt $l]compatible = \"$value\";\n"
		set out "$out[tt $l]memreg:reg = < $eram_base $eram_size >;\n"
		set out "$out[tt $l]device_type = \"memory\";\n"
		set out "$out[ts $l]\} ;"

		if {[expr $x + 1] < $count} {
			set out "$out\n"
		}
	}
	return $out
}

proc test {name args} {
	puts "$name [llength $args]"

		if {[llength $name] > 1} {
			foreach in $args {
				puts "$in"
			}
	}
}


proc gen_ppc405 {hwproc_handle l} {
	set out ""
	variable cpunumber
	set args [xget_hw_parameter_handle $hwproc_handle "*"]
	foreach arg $args {
		set arg_name [xget_value $arg "NAME"]
		set arg_name [string map -nocase {C_ ""} $arg_name]
		set arg_value [xget_value $arg "VALUE"]
#		set out "$out DEBUG $arg_name $arg_value\n"
		switch $arg_name {
			INSTANCE {
				set ppc_name $arg_value
			}
			default {}
		}
	}
	set out "$out[ts $l]cpus \{\n"
	set out "$out[tt $l]#size-cells = <0>;\n"
	set out "$out[tt $l]#cpus = < $cpunumber >;\n"
	set out "$out[tt $l]#address-cells = <1>;\n"
	set out "$out[tt $l]$ppc_name,405@$cpunumber \{\n"
	set cpunumber [expr $cpunumber + 1]
	set l [expr $l + 1]
#	set out "$out[tt $l]32-bit;\n"
#	set out "$out[tt $l]linux,boot-cpu;\n"
	set out "$out[tt $l]device_type = \"cpu\";\n"
	set out "$out[tt $l]reg = <0>;\n"
	set out "$out[tt $l]clock-frequency = <5f5e1000>;\n"
	set out "$out[tt $l]timebase-frequency = <1FCA055>;\n"
	set l [expr $l - 1]
	set out "$out[tt $l]\} ;\n"
	set out "$out[ts $l]\} ;"
	return $out
}


proc gen_microblaze {hwproc_handle l} {
	set out ""
	variable cpunumber
	set args [xget_hw_parameter_handle $hwproc_handle "*"]
	foreach arg $args {
		set arg_name [xget_value $arg "NAME"]
		set arg_name [string map -nocase {C_ ""} $arg_name]
		set arg_value [xget_value $arg "VALUE"]
#		set out "$out DEBUG $arg_name $arg_value\n"
		switch $arg_name {
			HW_VER {
				set mr_hw_ver $arg_value
			}
			CACHE_BYTE_SIZE {
				set mr_icache_size $arg_value
			}
			DCACHE_BYTE_SIZE {
				set mr_dcache_size $arg_value
			}
			INSTANCE {
				set mr_name $arg_value
			}
			default {
#				set out "$out-----name = \"$arg $arg_name\"=$arg_value;\n"
			}
		}
	}
	set out "$out[ts $l]cpus \{\n"
	set out "$out[tt $l]#size-cells = <0>;\n"
	set out "$out[tt $l]#cpus = < $cpunumber >;\n"
	set out "$out[tt $l]#address-cells = <1>;\n"
#	set out "$out[tt $l]$mr_name,$mr_hw_ver@$cpunumber \{\n"
	set out "$out[tt $l]$mr_name@$cpunumber \{\n"
	set cpunumber [expr $cpunumber + 1]
	set l [expr $l + 1]
#	set out "$out[tt $l]32-bit;\n"
#	set out "$out[tt $l]linux,boot-cpu;\n"
	set out "$out[tt $l]device_type = \"cpu\";\n"
	set out "$out[tt $l]reg = <0>;\n"
	set out "$out[tt $l]clock-frequency = <5f5e1000>;\n"
	set out "$out[tt $l]timebase-frequency = <1FCA055>;\n"

# vyborny test na nulovy pointer
	if { [llength $mr_icache_size] != 0 } {
		set mr_icache_size [format "%x" $mr_icache_size]
		set out "$out[tt $l]i-cache-line-size = <$mr_icache_size>;\n"
		set out "$out[tt $l]i-cache-size = <10>;\n"
	}
	if { [llength $mr_dcache_size] != 0 } {
		set mr_dcache_size [format "%x" $mr_dcache_size]
		set out "$out[tt $l]d-cache-line-size = <$mr_dcache_size>;\n"
		set out "$out[tt $l]d-cache-size = <10>;\n"
	}

#-----------------------------
# generating additional parameters
# the list of Microblaze parameters
	set param "PVR DEBUG_ENABLED FSL_LINKS UNALIGNED_EXCEPTIONS ILL_OPCODE_EXCEPTION IOPB_BUS_EXCEPTION DOPB_BUS_EXCEPTION DIV_ZERO_EXCEPTION FPU_EXCEPTION"
	if {[llength $param]} {
		if {[llength $param] > 0} {
			foreach i $param {
				set par_handle [xget_hw_parameter_handle $hwproc_handle "C_$i"]
				if {[llength $par_handle]} {
					set par_name [xget_hw_name $par_handle]
					set par_name [string map -nocase {"C_" ""} $par_name]
					set par_name [string tolower $par_name]
					set par_name [string map -nocase {"_" "-"} $par_name]
					set par_value [xget_hw_value $par_handle]
					set par_value [format "%x" $par_value]
					set out "$out[tt $l]xilinx,$par_name = <$par_value>;\n"
				}
			}
		}
	} else {
#		set out "$out parameters don't exist [llength $param]\n"
	}
#-----------------------------
	set l [expr $l - 1]
	set out "$out[tt $l]\} ;\n"
	set out "$out[ts $l]\} ;"
	return $out
}
# Test if is periphery bridge?
# 0 - bridge
# <> - no bridge
proc bridge_test {slave} {
	set name [xget_hw_name $slave]
	set value [xget_hw_value $slave]
#	puts "++++++$slave $name=$value +++++++++++++"
	switch $value {
		"plb2opb_bridge" -
		"opb2plb_bridge" -
		"opb_opb_lite" {
#			puts "----------bridge $value"
			set ret "0"

		}
		default {
#			puts "----------no bridge $value"
			set ret "1"
		}
	}
	return $ret
}

proc bus_bridge {slave l intc face} {
#FIXME - remove enter - clear variable out \n
	set out ""
#	puts "+++++++++++ $slave ++++++++"
	set bus_handle [xget_hw_busif_handle $slave $face]
#	puts "--$bus_handle"
	set mhs_handle [xget_hw_parent_handle $slave]
	set masters [xget_hw_connected_busifs_handle $mhs_handle [xget_hw_value $bus_handle] "master"]
	set slaves [xget_hw_connected_busifs_handle $mhs_handle [xget_hw_value $bus_handle] "slave"]
	set per "$slaves $masters"
#	puts "$per"
#compose peripherals & cleaning
	set slaves ""
	foreach slav $per {
		set sl [xget_hw_parent_handle $slav]
#		puts "-$slave $sl [xget_hw_name $slav] [xget_hw_value $slav] [xget_hw_name $sl]"
		if {[lsearch $slaves $sl] == -1} {
			set slaves "$slaves $sl"
		} else {
#			puts "the same periphery $sl"
		}
	}
#	puts "remove master periphery $slaves"
#remove bridge from list
	set position [lsearch $slaves $slave]
	set slaves [lreplace $slaves $position $position]
	variable periphery_array
# declare variable for bridges - because I need make a process with bridge after all slave peripherals
	set bridges_array ""
	append periphery_array "$slave "
#	puts "after removing master periphery $slaves"

#for all peripherals
	foreach slave $slaves {
#		puts "*$slave"
		if {[lsearch $periphery_array $slave] == -1} {
			if {[bridge_test $slave] == "1"} {
#				puts "no bridge $slave"
				set out "$out\n[gener_slave $slave $l $intc]"
				append periphery_array "$slave "
			} else {
				append bridges_array "$slave "
#				puts "bridge detected"
			}
		} else {
#			puts "periphery is in the list"
		}
#		if { [llength $data] != 0 } {
#			puts "$data"
#		}
	}
#	puts "$periphery_array ---- $bridges_array"
	foreach slave $bridges_array {
 		if {[lsearch $periphery_array $slave] == -1} {
			set position [lsearch $bridges_array $slave]
#			puts "position in array $position  [lreplace $bridges_array $position $position]"
			set $bridges_array [lreplace $bridges_array $position $position]
#			puts "1234 $periphery_array -- $bridges_array"
			set out "$out\n[gener_slave $slave $l $intc]"
#			puts "append periphery $slave"
			append periphery_array "$slave "
		}
	}
#	puts "5678 $periphery_array --- $bridges_array"
	return $out
}

